package gov.bnl.gums;

import gov.bnl.gums.account.AccountMapper;

import java.lang.ref.SoftReference;

import javax.persistence.*;

@Entity
public class Mapping {
	protected static String[] stateTypes = {"available", "assigned", "suspended"};
	protected int stateIndex = 0;
	
	// persistent fields
	protected long id;
	protected String dn;
	protected SiteUser siteUser = new SiteUser();
	protected SoftReference<AccountMapper> accountMapperRef = null;
	
	public Mapping() {}

	public Mapping(String dn, SiteUser siteUser) {
		assert(siteUser != null);
		this.dn = dn;
		this.siteUser = siteUser;
	}
	
	public Mapping(String dn, SiteUser siteUser, String state) {
		assert(siteUser != null);
		this.dn = dn;
		this.siteUser = siteUser;
		setState(state);
	}

	public boolean equals(Object o) {
		if (o == null)
			return false;
		else if (o instanceof Mapping) {
			Mapping mapping = (Mapping)o;
			assert(siteUser != null);
			if (siteUser==null)
				throw new RuntimeException("siteuser null");
			assert(((Mapping)o).getSiteUser() != null);
			if (dn==null && mapping.dn==null)
				return true;
			else if (dn==null || mapping.dn==null)
				return false;
			else 
				return dn.equals(mapping.getDn()) && siteUser.equals(mapping.getSiteUser());
		}
		else if (o instanceof SiteUser)
			return siteUser.equals((SiteUser)o);
		else if (o instanceof GridUser && dn!=null)
			return dn.equals((GridUser)o);
		else
			return false;
	}
	
	@ManyToOne
    @JoinColumn(name="accountMapper")
    public AccountMapper getAccountMapper() {
		return accountMapperRef == null ? null : accountMapperRef.get();
	}

	public String getDn() {
		return dn;
	}
	
    public SiteUser getSiteUser() {
		return siteUser;
	}
    
	public String getState() {
		return stateTypes[stateIndex];
	}

	public int hashCode() {
    	return dn!=null ? 10*dn.hashCode() : 0 + 10*dn.hashCode() + siteUser.hashCode();
    }

	public void setAccountMapper(AccountMapper accountMapper) {
		this.accountMapperRef = new SoftReference<AccountMapper>(accountMapper);
	}

	public void setDn(String dn) {
		this.dn = dn;
	}

	public void setSiteUser(SiteUser siteUser) {
		assert(siteUser != null):
		this.siteUser = siteUser;
	}

	public void setState(String state) {
		if (state==null)
			return;
    	for(int i=0; i<stateTypes.length; i++) {
    		if ( stateTypes[i].equalsIgnoreCase(state) ) {
    			stateIndex = i;
    			return;
    		}
    	}
    	throw new RuntimeException("Invalid state type: "+state);
	}

	public String toString() {
    	return dn + "," + siteUser.toString();
    }

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@SuppressWarnings("unused")
	private long getId() {
		return id;
	}

	@SuppressWarnings("unused")
	private void setId(long id) {
		this.id = id;
	}
}
