/*
 * AccountMapper.java
 *
 * Created on March 30, 2004, 5:56 PM
 */

package gov.bnl.gums.account;

import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.Entity;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;

import gov.bnl.gums.SiteUser;
import gov.bnl.gums.configuration.ConfigElement;
import gov.bnl.gums.configuration.Configuration;

/** 
 * Defines the logic with which a user will be mapped to a local account.
 * As of now, the logic is a simple certificate subject mapped to a user
 * account. In the future this interface will be extended to map the credential
 * of a full proxy (DN, vo, role, group) to a user and group account.
 *
 * @author  Gabriele Carcassi, Jay Packard
 */
@Entity
@Inheritance(strategy=InheritanceType.SINGLE_TABLE)
@DiscriminatorColumn(
    name="type",
    discriminatorType=DiscriminatorType.STRING
)
public abstract class AccountMapper extends ConfigElement {
	/**
	 * Create an account mapper object - empty constructor needed by XML Digestor
	 */
	public AccountMapper() {
    }
	
	/**
	 * Create an account mapper object with a given configuration and name
	 * 
	 * @param configuration
	 * @param name
	 */
	public AccountMapper(Configuration configuration, String name) {
		super(configuration, name);
    }
 	
	/**
     * Maps a grid user to a site user.
     * @param the GridUser
     * @return a user account (i.e. 'atlas').
     */
    public abstract SiteUser mapDn(String dn, boolean createNew);

}
