/*
 * GecosLdapAccountMapper.java
 *
 * Created on April 13, 2005, 4:21 PM
 */

package gov.bnl.gums.account;

import gov.bnl.gums.SiteUser;
import gov.bnl.gums.configuration.Configuration;
import gov.bnl.gums.util.CertToolkit;
import gov.bnl.gums.util.GecosMap;

import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;

import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchResult;
import javax.persistence.Entity;
import javax.persistence.Transient;

import org.apache.log4j.Logger;

/** 
 * Matches the DN with the account information retrieved from an LDAP server.
 *
 * @author Gabriele Carcassi, Jay Packard
 */
@Entity
public class GecosLdapAccountMapper extends LdapAccountMapper {
	static private Logger log = Logger.getLogger(GecosLdapAccountMapper.class);
    static private Map<String, GecosMap> gecosMaps = new Hashtable<String, GecosMap>();

    // persistent variables
    protected String gecosField = "gecos";
	
    public GecosLdapAccountMapper() {
    	super();
    }
    
    public GecosLdapAccountMapper(Configuration configuration, String name) {
    	super(configuration, name);
    }
	
    public String getGecosField() {
		return gecosField;
	}
    
    public SiteUser mapDn(String dn, boolean createIfDoesNotExist) {
        String[] nameSurname = CertToolkit.parseNameAndSurname(dn);
        GecosMap map = gecosMap();
        log.trace("GECOS findAccount. Name: " + nameSurname[0] + " - Surname: " + nameSurname[1] + " - GECOSMap: " + gecosMap());
        String account = map.findAccount(nameSurname[0], nameSurname[1]);
        return account!=null ? new SiteUser(account) : null;
    }
    
    @ConfigFieldAnnotation(label="Full Name Field", example="gecos")
	public void setGecosField(String gecosField) {
		this.gecosField = gecosField;
	}    

	protected GecosMap createMap() {
		Properties jndiProperties = getProperties();
		int nTries = 5;
		Exception lastException = null;
		int i = 0;
		for (; i < nTries; i++) {
			GecosMap map = new GecosMap();
			log.debug("Attempt " + i + " to retrieve map for '" + jndiLdapUrl + "'");
			try {
				DirContext jndiCtx = new InitialDirContext(jndiProperties);
				NamingEnumeration<SearchResult> nisMap = jndiCtx.search(peopleObject, "("+accountField+"=*)", null);
				log.trace("Server responded");
				while (nisMap.hasMore()) {
					SearchResult res = (SearchResult) nisMap.next();
					Attributes atts = res.getAttributes();
					String account = (String) atts.get(accountField).get();
					Attribute gecosAtt = atts.get(gecosField);
					if (gecosAtt != null) {
						String gecos = gecosAtt.get().toString();
						map.addEntry(account, gecos);
					} else {
						log.trace("Found user '" + account + "' with no GECOS field");
					}
				}
				jndiCtx.close();
				return map;
			} catch (javax.naming.NamingException ne) {
				log.warn("Error filling the maps for NIS "+jndiLdapUrl, ne);
				lastException = ne;
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					log.warn("Interrupted", e);
				}
			} catch (Exception e) {
				log.warn("Error filling the maps for NIS "+jndiLdapUrl, e);
				lastException = e;
				try {
					Thread.sleep(100);
				} catch (InterruptedException ie) {
					log.warn("Interrupted", e);
				}
			}
		}
		if (i == nTries) {
			throw new RuntimeException("Couldn't retrieve NIS maps from " + jndiLdapUrl, lastException);
		}
		return null;
	}

	protected GecosMap gecosMap() {
        synchronized (gecosMaps) {
            GecosMap map = (GecosMap) gecosMaps.get(getMapName());
            if (map != null) {
                if (map.isValid()) {
                    log.trace("Reusing GECOS map for '" + getMapName() +"'");
                    return map;
                } else {
                    log.trace("Invalidating expired GECOS map for '" + getMapName() +"'");
                    gecosMaps.remove(getMapName());
                }
            }
            log.debug("Creating new GECOS map for '" + getMapName() + "'");
            map = createMap();
            gecosMaps.put(getMapName(), map);
            return map;
        }
    }

	@Transient
	protected String getMapName() {
		return jndiLdapUrl;
	}
}
