/*
 * GroupAccountMapper.java
 *
 * Created on May 25, 2004, 2:10 PM
 */

package gov.bnl.gums.account;

import javax.persistence.Entity;
import javax.persistence.Transient;

import gov.bnl.gums.SiteUser;
import gov.bnl.gums.configuration.Configuration;

import org.apache.log4j.Logger;


/** 
 * An account mapping policy that maps all user to the same account.
 * <p>
 * To configure the policy one needs only to set the groupName property
 * as the desired group account.
 *
 * @author Gabriele Carcassi, Jay Packard
 */
@Entity
public class GroupAccountMapper extends AccountMapper {
	static private Logger log = Logger.getLogger(GroupAccountMapper.class);
    
	// persistent variables
	private String accountName;
    
    public GroupAccountMapper() {
    	super();
    }
    
    public GroupAccountMapper(Configuration configuration, String name) {
    	super(configuration, name);
    }
	
	public void clearCachedItems() {

	}
    
	public String getAccountName() {
		return accountName;
	}    
    
    public SiteUser mapDn(String dn, boolean createIfDoesNotExist) {       
        if (log.isDebugEnabled())
            log.debug("User " + dn + " mapped to account " + accountName);
        
        return new SiteUser(accountName);
    }

    @ConfigFieldAnnotation(label="Account", example="myAccount", help="the account to map to") 
    public void setAccountName(String accountName) {
    	if (this.accountName != null)
    		log.debug("account name changed from  " + this.accountName + " to " + accountName);
		this.accountName = accountName;
	}

}
