package gov.bnl.gums.account;

import javax.persistence.Entity;
import javax.persistence.Transient;

import gov.bnl.gums.Mapping;
import gov.bnl.gums.SiteUser;
import gov.bnl.gums.configuration.Configuration;

@Entity
public class LdapAccountPoolMapper extends AccountPoolMapper {
	protected LdapAccountMapper ldap = new LdapAccountMapper();
	
	// persistent variables
	protected String jndiLdapUrl;
	protected String ldapPrincipal;
	protected String ldapPassword;
	protected String dnField = "description";
	protected String accountField = "uid";
	protected String memberUidField = "memberUid";
	protected String gidNumberField = "gidNumber";
	protected String groupCnField = "cn";
	protected String peopleTree;
	protected String peopleObject = "ou=People";
	protected String peopleContext;
	protected String groupTree;
	protected String groupObject = "ou=Group";
	protected String groupContext;
	protected boolean synchGroups;

    public LdapAccountPoolMapper() {
    	super();
    }
    
    public LdapAccountPoolMapper(Configuration configuration, String name) {
    	super(configuration, name);
    }
	
	public String getAccountField() {
		return ldap.getAccountField();
	}
	
	public String getDnField() {
		return ldap.getDnField();
	}
	
	public String getGidNumberField() {
		return ldap.getGidNumberField();
	}
	
	public String getGroupCnField() {
		return ldap.getGroupCnField();
	}
	
	public String getGroupTree() {
		return ldap.getGroupTree();
	}
	
	public String getJndiLdapUrl() {
		return ldap.getJndiLdapUrl();
	}
	
	public String getMemberUidField() {
		return ldap.getMemberUidField();
	}
	
	public String getPeopleTree() {
		return ldap.getPeopleTree();
	}
	
	public boolean isSynchGroups() {
		return ldap.isSynchGroups();
	}
	
	public SiteUser mapDn(String dn, boolean createNew){
		SiteUser siteUser = super.mapDn(dn, createNew);
		boolean justCreated = false;
		synchronized(mappings) {
			if (siteUser == null && createNew) {
				for (Mapping m: mappings) {
					if (m.getDn() == null) {
						m.setDn(dn);
						siteUser = m.getSiteUser();
					}
				}
			}
			
			// Assign groups in ldap
			if (siteUser!=null && (justCreated || ldap.isSynchGroups())) {
				ldap.changeGroup(siteUser.getAccount(), primaryGroup);
				for (String g: secondaryGroups)
					ldap.addToSecondaryGroup(siteUser.getAccount(), g);
			}
		}
		return null;
	}
	
	@ConfigFieldAnnotation(label="LDAP UID Field", example="uid", help="(account UID field)")
	public void setAccountField(String accountField) {
		ldap.setAccountField(accountField);
	}
	
	@ConfigFieldAnnotation (label="LDAP Certificate DN Field", example="description", help="certificate DN field")
	public void setDnField(String dnField) {
		ldap.setDnField(dnField);
	}
	
	@ConfigFieldAnnotation(label="LDAP GID Number Field", example="gidNumber", help="(group ID number field in 'People' object)")
	public void setGidNumberField(String gidNumberField) {
		ldap.setGidNumberField(gidNumberField);
	}
	
	@ConfigFieldAnnotation(label="LDAP Group CN Field", example="cn", help="(group common name field)")
	public void setGroupCnField(String groupCnField) {
		ldap.setGroupCnField(groupCnField);
	}
	
	@ConfigFieldAnnotation(label="Group Tree", example="ou=Group,dc=usatlas,dc=bnl,dc=gov", help="(relative to context in LDAP URL - optional)")
	public void setGroupTree(String groupTree) {
		ldap.setGroupTree(groupTree);
	}
	
	@ConfigFieldAnnotation(label="JNDI LDAP URL", example="ldap://localhost/dc=usatlas,dc=bnl,dc=gov") 
	public void setJndiLdapUrl(String jndiLdapUrl) {
		ldap.setJndiLdapUrl(jndiLdapUrl);
	}
	
	@ConfigFieldAnnotation(label="Member Field", example="memberUid", help="field containing user ID in 'Group' object")
	public void setMemberUidField(String memberUidField) {
		ldap.setMemberUidField(memberUidField);
	}
	
	@ConfigFieldAnnotation(label="People Tree", example="ou=People,dc=usatlas,dc=bnl,dc=gov", help="relative to context in LDAP URL - optional")
	public void setPeopleTree(String peopleTree) {
		ldap.setPeopleTree(peopleTree);
	}
	
	@ConfigFieldAnnotation(label="Update group and email for every access")
	public void setSynchGroups(boolean synchGroups) {
		ldap.setSynchGroups(synchGroups);
	}
}
