/*
 * GUMSAPIImpl.java
 *
 * Created on November 1, 2004, 12:18 PM
 */

package gov.bnl.gums.admin;

import gov.bnl.gums.*;
import gov.bnl.gums.account.AccountMapper;
import gov.bnl.gums.account.AccountPoolMapper;
import gov.bnl.gums.account.ManualAccountMapper;
import gov.bnl.gums.configuration.Configuration;
import gov.bnl.gums.configuration.ConfigurationToolkit;
import gov.bnl.gums.userGroup.ManualUserGroup;
import gov.bnl.gums.userGroup.UserGroup;
import gov.bnl.gums.hostToGroup.HostToGroupMapping;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Iterator;

import org.apache.log4j.Logger;

/**
 * GUMSAPI implementation - handles authorization and wraps core logic - shouldn't do sophisticated logic here
 *
 * @author Gabriele Carcassi, Jay Packard
 */
public class GUMSAPIImpl implements GUMSAPI {
	static private GUMS gums;
	//private Logger log = Logger.getLogger(GUMSAPI.class);
	private Logger gumsAdminLog = Logger.getLogger(GUMS.gumsAdminLogName);
	private Logger siteAdminLog = Logger.getLogger(GUMS.siteAdminLogName);
	private boolean isInWeb = false;

	{
		try {
			Class.forName("javax.servlet.Filter");
			isInWeb = true;
		} catch (ClassNotFoundException e) {
			isInWeb = false;
		}
	}

	public void addAccountRange2(String accountPoolMapperName, String range) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				gums().getCoreLogic().addAccountRange(accountPoolMapperName, range);
				gumsAdminLog.info(logUserAccess() + "Added account range '" + range + "' to account mapper '" + accountPoolMapperName + "'");
				siteAdminLog.info(logUserAccess() + "Added account range '" + range + "' to account mapper '" + accountPoolMapperName + "'");
			}
			else {
				String message = logUserAccess() + "Unauthorized access to addAccountRange2 for '"+accountPoolMapperName+" "+range+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public void backupConfiguration(String name) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				Configuration conf = (Configuration)gums().getConfiguration().clone();
				conf.setName(name);
				gums().setConfiguration(conf);
				gumsAdminLog.info(logUserAccess() + "Backed up configuration '" + name + "'");
				siteAdminLog.info(logUserAccess() + "Backed up configuration '" + name + "'");
			}
			else {
				String message = logUserAccess() + "Unauthorized access to backupConfiguration";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public void beginTransaction() throws Exception {
		gums().beginTransaction();
	}

	public void commit() throws IOException, IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException, ClassNotFoundException {
		gums().commit();
	}
	
	public void rollback() {
		try {
			gums().rollback();
		} catch (Exception e) {
		}
	}
	
	public void deleteBackupConfiguration(String name) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				gums().removeBackupConfiguration(name);
			}
			else {
				String message = logUserAccess() + "Unauthorized access to deleteBackupConfiguration for '"+name+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}  
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}		
	}

	public String generateEmailMapfile(String hostname) {
		try {
			String map;
			beginTransaction();
			if (hasReadAllAccess(currentUser(), hostname)) {
				map = gums().getCoreLogic().generateGridMapfile(hostname, false, true, true);
				if (gumsAdminLog.isDebugEnabled())
					gumsAdminLog.debug(logUserAccess() + "Generated email mapfile for host '" + hostname + "': " + map);
				else
					gumsAdminLog.info(logUserAccess() + "Generated email mapfile for host '" + hostname + "'");
			} 
			else { 
				String message = logUserAccess() + "Unauthorized access to generateEmailMapfile for host '"+hostname+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}
			commit();
			return map;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public String generateFqanMapfile(String hostname) {
		try {
			String map;
			beginTransaction();
			if (hasReadAllAccess(currentUser(), hostname)) {
				map = gums().getCoreLogic().generateFqanMapfile(hostname, true);
				if (gumsAdminLog.isDebugEnabled())
					gumsAdminLog.debug(logUserAccess() + "Generated fqan mapfile for host '" + hostname + "': " + map);
				else
					gumsAdminLog.info(logUserAccess() + "Generated fqan mapfile for host '" + hostname + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to generateFqanMapfile for host '"+hostname+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}
			commit();
			return map;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}        

	public String generateGridMapfile(String hostname) {
		try {
			String map;
			beginTransaction();
			if (!gums().getConfiguration().isAllowGridmapFiles())
				throw new RuntimeException("Grid Mapfile generation has been disabled (probably to conserve pool accounts)");

			if (hasReadAllAccess(currentUser(), hostname)) {
				map = gums().getCoreLogic().generateGridMapfile(hostname, true, false, false);
				if (gumsAdminLog.isDebugEnabled())
					gumsAdminLog.debug(logUserAccess() + "Generated grid mapfile for host '" + hostname + "': " + map);
				else
					gumsAdminLog.info(logUserAccess() + "Generated grid mapfile for host '" + hostname + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to generateGridMapfile for host '"+hostname+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}
			
			commit();
			return map;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public String generateOsgUserVoMap(String hostname) {
		try {
			String map;
			beginTransaction();
			if (hasReadAllAccess(currentUser(), hostname)) {
				map = gums().getCoreLogic().generateOsgUserVoMap(hostname);
				commit();
				if (gumsAdminLog.isDebugEnabled())
					gumsAdminLog.debug(logUserAccess() + "Generated osg vo-user map for host '" + hostname + "': " + map);
				else
					gumsAdminLog.info(logUserAccess() + "Generated osg vo-user map for host '" + hostname + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to generateOsgUserVoMap for host '"+hostname+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}
			commit();
			return map;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public String generateVoGridMapfile(String hostname) {
		try {
			String map;
			beginTransaction();
			
			if (!gums().getConfiguration().isAllowGridmapFiles())
				throw new RuntimeException("Grid Mapfile generation has been disabled (probably to conserve pool accounts)");

			if (hasReadAllAccess(currentUser(), hostname)) {
				beginTransaction();
				map = gums().getCoreLogic().generateGridMapfile(hostname, true, true, false);
				commit();
				if (gumsAdminLog.isDebugEnabled())
					gumsAdminLog.debug(logUserAccess() + "Generated vo grid mapfile for host '" + hostname + "':" + map);
				else
					gumsAdminLog.info(logUserAccess() + "Generated vo grid mapfile for host '" + hostname + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to generateVoGridMapfile for host '"+hostname+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}
			
			
			commit();
			
			return map;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public Collection<String> getBackupNames() {
		try {
			beginTransaction();
			Collection<String> backupConfigDates = null;
			if (hasReadAllAccess(currentUser(), null)) {
				backupConfigDates = gums().getBackupConfigurationNames();
			}
			else {
				String message = logUserAccess() + "Unauthorized access to getBackupConfigDates";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();
			}    	
			commit();
			return backupConfigDates;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public Configuration getConfiguration() {
		try {
			if (hasReadAllAccess(currentUser(), null) && isInWeb) {
				return gums().getConfiguration();
			}
			else {
				String message = logUserAccess() + "Unauthorized access to getConfiguration";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();		
			}
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}
	}

    public String getOutOfDateFqans(int hoursSinceLastUpdate) {
    	try {
    		if (hasReadAllAccess(currentUser(), null)) {
				return gums().getCoreLogic().getOutOfDateFqans(hoursSinceLastUpdate);
			}
			else {
				String message = logUserAccess() + "Unauthorized access to getOutOfDateFqans";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();		
			}
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}   	
    }
	
	public String getPoolAccountAssignments(String accountPoolMapperName) {
		try {
			String retVal;
			beginTransaction();
			if (hasReadAllAccess(currentUser(), null)) {
				if (gums().getConfiguration().getAccountMapper(accountPoolMapperName) instanceof AccountPoolMapper)
					retVal = ((AccountPoolMapper)gums().getConfiguration().getAccountMapper(accountPoolMapperName)).getAssignments();
				else
					retVal = "";
			}
			else {
				String message = logUserAccess() + "Unauthorized access to getPoolAccountAssignments for '"+accountPoolMapperName+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();	
			}
			commit();
			return retVal;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}		
	}

	public String getVersion() {
		return GUMS.getVersion();
	}

	public void manualGroupAdd2(String manualUserGroupName, String userDN) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				((ManualUserGroup)gums().getConfiguration().getUserGroup(manualUserGroupName)).addMember(new GridUser(userDN));
				gumsAdminLog.info(logUserAccess() + "Added to user group '" + manualUserGroupName + "' user '" + userDN + "'");
				siteAdminLog.info(logUserAccess() + "Added to user group '" + manualUserGroupName + "' user '" + userDN + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to manualGroupAdd2 for user group '" + manualUserGroupName + "' user '" + userDN + "'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();	
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}		
	}

	public void manualGroupAdd3(String manualUserGroupName, String userDN, String fqan, String email) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				((ManualUserGroup)gums().getConfiguration().getUserGroup(manualUserGroupName)).addMember(new GridUser(userDN, fqan, email, false));
				gumsAdminLog.info(logUserAccess() + "Added to user group '" + manualUserGroupName + "' user '" + userDN + "' fqan '" + fqan + "' email '" + email + "'");
				siteAdminLog.info(logUserAccess() + "Added to user group '" + manualUserGroupName + "' user '" + userDN + "' fqan '" + fqan + "' email '" + email + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to manualGroupAdd3 for user group '" + manualUserGroupName + "' user '" + userDN + "' fqan '" + fqan + "' email '" + email + "'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);
				throw new AuthorizationDeniedException();	
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}		
	}

	public void manualGroupRemove2(String manualUserGroupName, String userDN) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				((ManualUserGroup)gums().getConfiguration().getUserGroup(manualUserGroupName)).removeMember(new GridUser(userDN));
				gumsAdminLog.info(logUserAccess() + "Removed from user group '" + manualUserGroupName + "'  user '" + userDN + "'");
				siteAdminLog.info(logUserAccess() + "Removed from user group '" + manualUserGroupName + "'  user '" + userDN + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to manualGroupRemove2 for user group '" + manualUserGroupName + "' user '" + userDN + "'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);				
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}			
	}

	public void manualGroupRemove3(String manualUserGroupName, String userDN, String fqan) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				((ManualUserGroup)gums().getConfiguration().getUserGroup(manualUserGroupName)).removeMember(new GridUser(userDN, fqan, false));
				gumsAdminLog.info(logUserAccess() + "Removed from user group '" + manualUserGroupName + "'  user '" + userDN + "' fqan '" + fqan + "'");
				siteAdminLog.info(logUserAccess() + "Removed from user group '" + manualUserGroupName + "'  user '" + userDN + "' fqan '" + fqan + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to manualGroupRemove2 for user group '" + manualUserGroupName + "' user '" + userDN + "' fqan '"+fqan+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);				
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}			
	}

	public void manualMappingAdd2(String manualAccountMapperName, String userDN, String account) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				AccountMapper accountMapper = gums().getConfiguration().getAccountMapper(manualAccountMapperName);
				if (accountMapper==null)
					throw new RuntimeException("Account Mapper '" + manualAccountMapperName + "' does not exist");
				if (!(accountMapper instanceof ManualAccountMapper))
					throw new RuntimeException("Account Mapper '" + manualAccountMapperName + "' is not an account pool mapper");
				ManualAccountMapper manualAccountMapper = new ManualAccountMapper();

				manualAccountMapper.createMapping(userDN, new SiteUser(account));

				gumsAdminLog.info(logUserAccess() + "Added mapping to account mapper '" + manualAccountMapperName + "' for user '" + userDN + "' to account '" + account + "'");
				siteAdminLog.info(logUserAccess() + "Added mapping to account mapper '" + manualAccountMapperName + "' for user '" + userDN + "' to account '" + account + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to manualMappingAdd2 for account mapper '" + manualAccountMapperName + "' user '" + userDN + "' account '"+account+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);				
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}		
	}

	public void manualMappingRemove2(String manualAccountMapperName, String userDN) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				AccountMapper accountMapper = gums().getConfiguration().getAccountMapper(manualAccountMapperName);
				if (accountMapper==null)
					throw new RuntimeException("Account Mapper '" + manualAccountMapperName + "' does not exist");
				if (!(accountMapper instanceof ManualAccountMapper))
					throw new RuntimeException("Account Mapper '" + manualAccountMapperName + "' is not a manaul pool mapper");
				ManualAccountMapper manualAccountMapper = new ManualAccountMapper();

				manualAccountMapper.removeMapping(userDN);

				gumsAdminLog.info(logUserAccess() + "Removed mapping from account mapper '" + manualAccountMapperName + "' for user '" + userDN + "'");
				siteAdminLog.info(logUserAccess() + "Removed mapping from account mapper '" + manualAccountMapperName + "' for user '" + userDN + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to manualMappingRemove2 for user group '" + manualAccountMapperName + "' user '" + userDN + "'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);	
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}		
	}

	public String mapAccount(String accountName) {
		try {
			String DNs;
			beginTransaction();
			if (hasReadAllAccess(currentUser(), null)) {
				DNs = gums().getCoreLogic().mapAccount(accountName);
				gumsAdminLog.info(logUserAccess() + "Mapped the account '" + accountName + "' to '" + DNs + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to mapAccount for account '" + accountName + "'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);	
				throw new AuthorizationDeniedException();
			}
			commit();
			return DNs;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public String mapUser(String hostname, String userDN, String fqan) {
		try {
			String account = null;
			beginTransaction();
			if ( (hasReadSelfAccess(currentUser()) && currentUser().compareDn(userDN)==0) || hasReadAllAccess(currentUser(), hostname)) {
				GridUser user = new GridUser(userDN, fqan);
				boolean isUserBanned = gums().getCoreLogic().isUserBanned(user);
				if (!isUserBanned) {
					SiteUser siteUser = gums().getCoreLogic().map(hostname, user, false);
					account = siteUser.getAccount();
				}
				if (account==null && isUserBanned) {
					String message = logUserAccess() + "Mapped on host '" + hostname + "' the banned user '" + userDN + "' / '" + fqan + "' to '" + account + "'";
					gumsAdminLog.warn(message);
					siteAdminLog.warn(message);	
				}
				else
					gumsAdminLog.info(logUserAccess() + "Mapped on host '" + hostname + "' the user '" + userDN + "' / '" + fqan + "' to '" + account + "'");
			} else {
				String message = logUserAccess() + "Unauthorized access to mapUser for '" + hostname + "' from user '" + userDN + "' / '" + fqan + "'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);	
				throw new AuthorizationDeniedException();
			}
			commit();
			return account;
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public void removeAccountRange(String accountPoolMapperName, String range) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				gums().getCoreLogic().removeAccountRange(accountPoolMapperName, range);
				gumsAdminLog.info(logUserAccess() + "Removed account range  '" + range + "' from account mapper '" + accountPoolMapperName + "'");
				siteAdminLog.info(logUserAccess() + "Removed accounts range  '" + range + "' from account mapper '" + accountPoolMapperName + "'");
			}
			else {
				String message = logUserAccess() + "Unauthorized access to removeAccountRange for '"+accountPoolMapperName+" "+range+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);	
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public void restoreConfiguration(String name) throws Exception {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				Configuration configuration = gums().restoreConfiguration(name);
				gumsAdminLog.debug("Configuration was set: " + configuration.toXml());
				siteAdminLog.debug("Configuration was set: " + configuration.toXml());
			}
			else {
				String message = logUserAccess() + "Unauthorized access to restoreConfiguration for '"+name+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);	
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}		
	}
	
	public void setConfiguration(Configuration configuration) throws Exception {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				gums().setConfiguration(configuration);
				gumsAdminLog.debug("Configuration was set: " + configuration.toXml());
				siteAdminLog.debug("Configuration was set: " + configuration.toXml());
			}
			else {
				String message = logUserAccess() + "Unauthorized access to setConfiguration";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);	
				throw new AuthorizationDeniedException();
			}
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}					
	}

	public void unassignAccountRange(String accountPoolMapperName, String range) {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				gums().getCoreLogic().unassignAccountRange(accountPoolMapperName, range);
				gumsAdminLog.info(logUserAccess() + "Unassigned account range '" + range + "' from account mapper '" + accountPoolMapperName + "'");
				siteAdminLog.info(logUserAccess() + "Unassigned account range  '" + range + "' from account mapper '" + accountPoolMapperName + "'");
			}
			else {
				String message = logUserAccess() + "Unauthorized access to unassignAccountRange for '"+accountPoolMapperName+" "+range+"'";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);	
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}		
	}

	public void updateGroups() {
		try {
			beginTransaction();
			if (hasWriteAccess(currentUser())) {
				gums().getConfiguration().updateUserGroupMembers();
				gumsAdminLog.info(logUserAccess() + "Groups updated");
				siteAdminLog.info(logUserAccess() + "Groups updated");
			} else {
				String message = logUserAccess() + "Unauthorized access to updateGroups";
				gumsAdminLog.warn(message);
				siteAdminLog.warn(message);	
				throw new AuthorizationDeniedException();
			}
			
			commit();
		} catch (Exception e) {
			rollback();
			throw new RuntimeException(e.getMessage());
		}
	}

	public String getCurrentDn() {
		return currentUser().getDn();
	}

	// Depricated
	public String generateGrid3UserVoMap(String hostname) {
		return generateOsgUserVoMap(hostname);
	}
	
	public void manualGroupAdd(String persistanceFactory, String group, String userDN) {
		manualGroupAdd2(group, userDN);
	}

	public void manualGroupRemove(String persistanceFactory, String group, String userDN) {
		manualGroupRemove2(group, userDN);
	}

	public void manualMappingAdd(String persistanceFactory, String group, String userDN, String account) {
		manualMappingAdd2(group, userDN, account);
	}

	public void manualMappingRemove(String persistanceFactory, String group, String userDN) {
		manualMappingRemove2(group, userDN);
	}

	public void poolAddAccount(String persistanceFactory, String group, String accountName) {
		addAccountRange2(group, accountName + accountName.substring(accountName.indexOf('-')));
	}

	////////////

	private GridUser currentUser() {
		if (!isInWeb) return null;
		String DN = CertCache.getUserDN();
		if (DN != null) {
			return new GridUser(DN);
		} else {
			return null;
		}
	}

	private GUMS gums() throws FileNotFoundException, IOException, IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException, ClassNotFoundException {
		if (gums == null)
			gums = new GUMS();
		return gums;
	}

	private boolean hasReadAllAccess(GridUser user, String hostname) throws Exception {
		if (user == null) 
			return false;
		if (hostname!=null && ConfigurationToolkit.testHostDNPattern.matcher(hostname).matches())
			return true;
		Collection<UserGroup> readAllUserGroups;
		if ((readAllUserGroups = gums().getConfiguration().getReadAllUserGroups()) != null) {
			Iterator<UserGroup> it = readAllUserGroups.iterator();
			while (it.hasNext()) {
				UserGroup userGroup = it.next();
				if (userGroup.isMember(user))
					return true;
			}
		}
		// return true if user certificate (issuer in reality - see CertToolkit) matches host certificate
		if (hostname!=null && user.getDn().indexOf(hostname) != -1)
			return true;
		// return true if user certificate (issuer in reality - see CertToolkit) matches any of the hostToGroupMappings
		if (hostToGroupMapping(user.getDn()) != null)
			return true;  
		return false;
	}

	private boolean hasReadSelfAccess(GridUser currentUser) throws Exception {
		if (currentUser == null) 
			return false;
		if (gums().getConfiguration().getReadSelfUserGroups() == null)
			return false;
		Collection<UserGroup> readSelfUserGroups = gums().getConfiguration().getReadSelfUserGroups();
		Iterator<UserGroup> it = readSelfUserGroups.iterator();
		while (it.hasNext()) {
			UserGroup userGroup = it.next();
			if (userGroup.isMember(currentUser))
				return true;
		}
		return false;
	}

	private boolean hasWriteAccess(GridUser user) throws Exception {
		if (user == null) 
			return false;
		if (gums().getConfiguration().getWriteUserGroups() == null)
			return false;
		Collection<UserGroup> writeUserGroups = gums().getConfiguration().getWriteUserGroups();
		Iterator<UserGroup> it = writeUserGroups.iterator();
		while (it.hasNext()) {
			UserGroup userGroup = it.next();
			if (userGroup.isMember(user)) 
				return true;
		}
		return false;
	}

	private HostToGroupMapping hostToGroupMapping(String hostname) {
		try {
			Collection<HostToGroupMapping> hostToGroupMappers = gums().getConfiguration().getHostToGroupMappings();
			Iterator<HostToGroupMapping> it = hostToGroupMappers.iterator();
			while (it.hasNext()) {
				HostToGroupMapping hostToGroupMapper = it.next();
				if (hostToGroupMapper.isInGroup(hostname)) {
					return hostToGroupMapper;
				}
			}
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}		
		return null;
	}

	private String logUserAccess() {
		if (currentUser() == null) {
			return "No AuthN - ";
		} else {
			return currentUser() + " - ";
		}
	}

}
