/*
 * CertificateHostGroup.java
 *
 * Created on May 10, 2005, 3:56 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package gov.bnl.gums.hostToGroup;

import gov.bnl.gums.configuration.Configuration;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.persistence.Entity;
import javax.persistence.Transient;

/**
 * Represent a set of services identified by a wildcard on their CN or DN.
 * 
 * @author Gabriele Carcassi, Jay Packard
 */
@Entity
public class CertificateHostToGroupMapping extends HostToGroupMapping {
	private List<String> regexs;

	// persistent variables
	protected String cn;
	protected String dn;

	/**
	 * Create a new CertificateHostToGroupMapping object.
	 */
	public CertificateHostToGroupMapping() {
		super();
	}

	/**
	 * Create a new CertificateHostToGroupMapping object.
	 * 
	 * @param configuration
	 */
	public CertificateHostToGroupMapping(Configuration configuration) {
		super(configuration, null);
	}

	/**
	 * Retrieves the wildcard that will be used to match the CN.
	 * 
	 * @return The wildcard (i.e. '*.mycompany.com').
	 */
	public String getCn() {
		return this.cn;
	}

	/**
	 * Retrieves the wildcard that will be used to match the DN.
	 * 
	 * @return The wildcard (i.e.
	 *         '/DC=org/DC=doegrids/OU=Services/CN=*.mycompany.com').
	 */
	public String getDn() {
		return this.dn;
	}

	public boolean isInGroup(String hostname) {
		Iterator<String> iter = regexs.iterator();
		while (iter.hasNext()) {
			if (hostname.matches((String) iter.next()))
				return true;
		}
		return false;
	}

	/**
	 * Changes the wildcard that will be used to match the CN(s).
	 * 
	 * @param cn
	 *            The new wildcard (i.e. '*.mycompany.com, *othercompany.com').
	 */
	@ConfigFieldAnnotation(label="CNs", example="*.host1.com, *.host2.com", help="only requests from matching hosts are accepted")
	public void setCn(String cn) {
		this.cn = cn;
		updateRegEx();
		if (cn!=null)
			setName(cn);
	}

	/**
	 * Changes the wildcard that will be used to match the DN(s).
	 * 
	 * @param wildcard
	 *            The new wildcard (i.e.
	 *            '/DC=org/DC=doegrids/OU=Services/CN=*.mycompany.com,
	 *            /DC=org/DC=doegrids/OU=Services/CN=*.othercompany.com').
	 */
	@ConfigFieldAnnotation(label="DNs", example="*.host1.com, *.host2.com", help="only requests from matching host DNs are accepted")
	public void setDn(String dn) {
		this.dn = dn;
		updateRegEx();
		if (dn!=null)
			setName(dn);
	}
	
	private void updateRegEx() {
		regexs = new ArrayList<String>();
		if (cn != null) {
			StringTokenizer tokens = new StringTokenizer(cn, ",");
			while (tokens.hasMoreTokens()) {
				String regex = tokens.nextToken();
				regex = regex.trim();
				regex = regex.replaceAll("\\.", "\\.");
				regex = regex.replaceAll("\\*", "[^\\./=]*");
				regexs.add("(/[^=]*=[^=]*)*/CN=" + regex + "(/[^=]*=[^=]*)*");
			}
		}
		if (dn != null) {
			StringTokenizer tokens = new StringTokenizer(dn, ",");
			while (tokens.hasMoreTokens()) {
				String regex = tokens.nextToken();
				regex = regex.trim();
				regex = regex.replaceAll("\\.", "\\.");
				regex = regex.replaceAll("\\*", "[^\\./=]*");
				regexs.add(regex);
			}
		}
	}
}
