/*
 * HibernatePersistenceFactory.java
 *
 * Created on June 15, 2005, 4:42 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package gov.bnl.gums.persistence;
import gov.bnl.gums.GridUser;
import gov.bnl.gums.configuration.Configuration;
import gov.bnl.gums.configuration.ConfigurationToolkit;
import gov.bnl.gums.userGroup.ManualUserGroup;

import java.util.Collection;
import java.util.Properties;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.FlushModeType;
import javax.persistence.NoResultException;
import javax.persistence.Persistence;

import org.apache.log4j.Logger;
import org.hibernate.*;


/**
 *
 * @author Gabriele Carcassi, Jay Packard
 */
public class HibernatePersistenceEngine implements PersistenceEngine {
	private Logger log = Logger.getLogger(HibernatePersistenceEngine.class);
	protected EntityManagerFactory entityManagerFactory;
	protected ThreadLocal<EntityManager> entityManager = new ThreadLocal<EntityManager>();
	protected SessionFactory sessions;
	protected Properties properties;

	/**
	 * Create a new hibernate persistence factory.  This empty constructor is needed by the XML Digester.
	 */
	public HibernatePersistenceEngine(Properties properties) {
		this.properties = properties;
		log.trace("HibernatePersistenceFactory instantiated");
		entityManagerFactory = Persistence.createEntityManagerFactory("gums", properties);
	}

	public boolean contains(Object obj) {
		return getEntityManager().contains(obj);
	}

	public void finalize() {
		try {
			if (entityManager!=null) {
				log.trace("Closing entity manager...");
				getEntityManager().close();
				log.trace("Closed entity manager.");
				entityManager = null;
			}
		} catch (HibernateException e) {
			log.error("Couldn't close hibernate sessions: " + e.getMessage());
		}
	}

	public void beginTransaction() {
		if (!getEntityManager().getTransaction().isActive()) {
			getEntityManager().getTransaction().begin();
		}
	}

	public void commit() {
		getEntityManager().getTransaction().commit();
	}
	
	public void rollback() {
		getEntityManager().getTransaction().rollback();
	}

	public void removeBackupConfiguration(String name) {
		getEntityManager().createQuery("removeBackupConfiguration").setParameter("name", name);
	}

	@SuppressWarnings("unchecked")
	public Collection<String> retrieveBackupConfigurationNames() {
		javax.persistence.Query q = getEntityManager().createQuery("select c.name from Configuration as c where c.name = null");
		return (Collection<String>)q.getResultList();
	}

	public Configuration retrieveCurrentConfiguration() {
		EntityManager entityManager = getEntityManager();
		javax.persistence.Query q = entityManager.createQuery("from Configuration c where c.name = null");
		Configuration conf;
		try {
			conf = (Configuration)q.getSingleResult();
		} catch (NoResultException e) {
			conf = new Configuration();
			
	    	// add admins usergroup if needed
	    	if (conf.getUserGroup("admins")==null) {
	    		ManualUserGroup userGroup = new ManualUserGroup(conf, "admins");
	    		userGroup.setAccess("write");
	    		// TODO: remove line below
	    		userGroup.addMember(new GridUser("/DC=org/DC=doegrids/OU=People/CN=John R. Hover 47116"));
	    	}

			// Add test user and test configuration
	    	if (conf.getGroupToAccountMapping("_test")==null)
	    		ConfigurationToolkit.insertTest(conf);
			
			entityManager.persist(conf);
		}
		return conf;
	}

	public Configuration restoreConfiguration(String name) {
		EntityManager entityManager = getEntityManager();

		// retrieve backup configuration
		javax.persistence.Query q = entityManager.createQuery("from Configuration where name = :name").setParameter("name", name);
		Configuration conf = (Configuration)q.getSingleResult();
		
		// clone
		conf = (Configuration)conf.clone();
		
		// remove current configuration
		entityManager.remove(retrieveCurrentConfiguration());

		// set new configuration
		conf.setCurrent();
		entityManager.persist(conf);
		
		return conf;
	}

	public void setConfiguration(Configuration conf) {
		Configuration currentConfig = retrieveCurrentConfiguration();
		if (currentConfig != null)
			getEntityManager().remove(currentConfig);
		getEntityManager().persist(conf);
	}

	protected EntityManager getEntityManager() {
		if (entityManager.get() == null) {
			EntityManager em = entityManagerFactory.createEntityManager();
			em.setFlushMode(FlushModeType.COMMIT);
			entityManager.set(em);
		}

		return entityManager.get();
	}

}
