/*
 * PersistanceManager.java
 *
 * Created on June 3, 2009, 4:46 PM
 */

package gov.bnl.gums.persistence;

import gov.bnl.gums.configuration.Configuration;

import java.util.Collection;

/** Represent an engine for all the classes that take care of the persistance of
 * the configuration and its AccountMappers, the UserGroups or any other components.
 * <p>
 * Implementing a new PersistanceManager class allows to complete redefined how
 * the user, group and accounting information is stored. This will allow to keep
 * the information integrated in the site accounting system, being it any RDBMS,
 * LDAP, or even a combination of these.
 * <p>
 * It also allow to use different mapping technologies, allowing to use either
 * entity beans, simple JDBC, Hibernate or any other favorite technology.
 *
 * @author Jay Packard
 */
public interface PersistenceEngine {
	void beginTransaction();

	void commit();

	void rollback();

	void removeBackupConfiguration(String name);
	
	Collection<String> retrieveBackupConfigurationNames();

	Configuration retrieveCurrentConfiguration();
	
	Configuration restoreConfiguration(String name);

	void setConfiguration(Configuration conf);
}
