package gov.bnl.gums.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

public class URIToolkit {
	static class CAFilter implements FilenameFilter {
		public boolean accept(File dir, String name) {
			return name.endsWith(".0");
		}
	}

	static class CRLFilter implements FilenameFilter {
		public boolean accept(File dir, String name) {
			return name.endsWith(".crl_url");
		}
	}
	
    static public String parseUri(String uri, String tempFile) throws Exception {
    	FileWriter fileWriter = null;
		InputStream inputStream = null;
    	try {
    		inputStream = getInputStream(uri);
			
			// write to temp file and string buffer
			StringBuffer buffer = new StringBuffer();
			if (tempFile != null)
				fileWriter = new FileWriter(new File(tempFile));
			int ch;
			while ((ch = inputStream.read()) != -1) {
				buffer.append((char)ch);
				if (tempFile != null)
					fileWriter.append((char)ch);
			}
			
			return buffer.toString();
		} finally {
			if (inputStream != null)
				inputStream.close();
			if (fileWriter != null)
				fileWriter.close();
		}
	}
    
    static public InputStream getInputStream(String uri) throws Exception {
		InputStream inputStream = null;
		if (uri.startsWith("https://")) {
			throw new Exception("https not yet supported");
		}
		else if (uri.startsWith("http")) {
			URL url = new URL(uri);
			URLConnection connection = url.openConnection();
			inputStream = connection.getInputStream();
		}
		else if (uri.startsWith("file:")) {
			inputStream = new FileInputStream(uri.substring(5));	
		}
		else {
			String message = "Invalid URI format: " + uri;
			throw new Exception(message);
		}
		return inputStream;
	}
}
