/*
 * GecosAccountMapperTest.java
 * JUnit based test
 *
 * Created on May 10, 2005, 1:35 PM
 */

package gov.bnl.gums.account;

import gov.bnl.gums.SiteUser;
import gov.bnl.gums.util.CertToolkit;
import gov.bnl.gums.util.GecosMap;
import gov.bnl.gums.configuration.Configuration;
import junit.framework.*;

/**
 *
 * @author carcassi
 */
public class GecosLdapAccountMapperTest extends TestCase {
    GecosLdapAccountMapper mapper;
    
    public GecosLdapAccountMapperTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
         mapper = new GecosAccountMapperImpl(new Configuration(), "myAccountMapper");
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(GecosLdapAccountMapperTest.class);
        
        return suite;
    }
    
    public void testMapUser() {
        assertEquals(new SiteUser("jsmith"), mapper.mapDn("/DC=org/DC=griddev/OU=People/CN=John Smith", true));
        assertNull(mapper.mapDn("/DC=org/DC=griddev/OU=People/CN=Evil Person", true));
    }

    public void testParsing() {
        String dn = "/DC=org/DC=griddev/OU=People/CN=John Smith Jr.";
        String[] name = CertToolkit.parseNameAndSurname(dn);
        assertEquals("John", name[0]);
        assertEquals("Smith", name[1]);
        name = CertToolkit.parseNameAndSurname("/DC=org/DC=griddev/OU=People/CN=Jane Doe 12345");
        assertEquals("Jane", name[0]);
        assertEquals("Doe", name[1]);
        name = CertToolkit.parseNameAndSurname("/DC=org/DC=griddev/OU=People/CN=John W. Smith Jr. 12345");
        assertEquals("John", name[0]);
        assertEquals("Smith", name[1]);
    }
    
    public void testCheckSurname() {
        assertTrue(CertToolkit.checkSurname("Smith"));
        assertFalse(CertToolkit.checkSurname("12345"));
        assertFalse(CertToolkit.checkSurname("Jr."));
    }

    private class GecosAccountMapperImpl extends GecosLdapAccountMapper {

        GecosAccountMapperImpl() {
            super();
        }

        GecosAccountMapperImpl(Configuration configuration, String name) {
            super(configuration, name);
        }
        
        protected GecosMap createMap() {
            GecosMap map = new GecosMap();
            map.addEntry("jsmith", "John Smith");
            return map;
        }

        protected java.lang.String getMapName() {
            return getName();
        }
        
        public AccountMapper clone(Configuration configuration) {
        	return null;
        }
    }

    
}
