/*
 * GridMapfileGeneratorTest.java
 * JUnit based test
 *
 * Created on March 24, 2004, 2:44 PM
 */

package gov.bnl.gums.configuration;

import gov.bnl.gums.userGroup.*;
import gov.bnl.gums.util.StringUtil;
import gov.bnl.gums.account.*;
import gov.bnl.gums.hostToGroup.*;
import gov.bnl.gums.groupToAccount.GroupToAccountMapping;
import gov.bnl.gums.*;

import java.util.*;

import junit.framework.*;

/**
 *
 * @author carcassi
 */
public class ConfigurationTest extends TestCase {
    
    public ConfigurationTest(java.lang.String testName) {
        super(testName);
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(ConfigurationTest.class);
        return suite;
    }
    
    public void testSimpleConfiguration() {
        Configuration conf = createSimpleConfiguration();
        HostToGroupMapping host2GroupMapping = (HostToGroupMapping) conf.getHostToGroupMappings().get(0);
        assertNotNull(host2GroupMapping);
        List<String> groupToAccountMappings = StringUtil.toList(host2GroupMapping.getGroupToAccountMappings());
        assertNotNull(groupToAccountMappings);
        assertEquals(1, groupToAccountMappings.size());
        GroupToAccountMapping gMap = conf.getGroupToAccountMapping( (String)groupToAccountMappings.get(0) );
        assertTrue((conf.getUserGroup( StringUtil.toList(gMap.getUserGroups()).get(0)) ).isMember(new GridUser("/DC=org/DC=griddev/OU=People/CN=John Smith", null)));
        assertEquals(new SiteUser("jsmith"), conf.getAccountMapper( StringUtil.toList(gMap.getAccountMappers()).get(0) ).mapDn("/DC=org/DC=griddev/OU=People/CN=John Smith", true));
    }
    
    public static Configuration createSimpleConfiguration() {
        Configuration conf = new Configuration();

        GroupToAccountMapping gMap = new GroupToAccountMapping(conf, "mockGroup");
        gMap.setAccountingVoSubgroup("mock");
        gMap.setAccountingVo("mock");
        
        ManualUserGroup userGroup = new ManualUserGroup(conf, "mockUserGroup");
        userGroup.addMember(new GridUser("/DC=org/DC=griddev/OU=People/CN=John Smith", null));
        gMap.addUserGroup(userGroup.getName());
        
        ManualAccountMapper accountMapper = new ManualAccountMapper(conf, "mockAccountMapper");
        accountMapper.createMapping("/DC=org/DC=griddev/OU=People/CN=John Smith", new SiteUser("jsmith"));
        gMap.addAccountMapper(accountMapper.getName());

        CertificateHostToGroupMapping hMap = new CertificateHostToGroupMapping(conf);
        hMap.addGroupToAccountMapping(gMap.getName());
        
        return conf;
    }
    
	public void testSingleUserGroupCopy() {
        Configuration conf = new Configuration();
        GroupToAccountMapping gMap = new GroupToAccountMapping(conf, "group1");
        LDAPUserGroup userGroup = new LDAPUserGroup(conf, "userGroup1");
		userGroup.setPeopleTree("ou=People,query");
		userGroup.setServer("server");
		gMap.addUserGroup(userGroup.getName());
		ManualAccountMapper accountMapper = new ManualAccountMapper(conf, "mockAccountMapper");
        gMap.addAccountMapper(accountMapper.getName());
        gMap = new GroupToAccountMapping(conf, "group2");
        userGroup = new LDAPUserGroup(conf, "userGroup2");
        userGroup.setPeopleTree("ou=People,query");
        userGroup.setServer("server");
        gMap.addUserGroup(userGroup.getName());
        accountMapper = new ManualAccountMapper(conf, "mockAccountMapper2");
        gMap.addAccountMapper(accountMapper.getName());
        assertEquals(2, conf.getGroupToAccountMappings().values().size());
        assertEquals(1, StringUtil.toList(gMap.getUserGroups()).size());
    }
    
    public void testMergeConfiguration() {
    	Configuration conf = new Configuration();
        GroupToAccountMapping gMap = new GroupToAccountMapping(conf, "mockGroup");
        gMap.setSource("testSource");
        gMap.setAccountingVoSubgroup("mock");
        gMap.setAccountingVo("mock");
        ManualUserGroup userGroup = new ManualUserGroup(conf, "mockUserGroup");
        userGroup.setSource("testSource");
        userGroup.addMember(new GridUser("/DC=org/DC=griddev/OU=People/CN=John Smith", null));
        gMap.addUserGroup(userGroup.getName());
        AccountMapper accountMapper = new ManualAccountMapper(conf, "mockAccountMapper");
        accountMapper.setSource("testSource");
        gMap.addAccountMapper(accountMapper.getName());
        CertificateHostToGroupMapping hMap = new CertificateHostToGroupMapping(conf);
        hMap.setCn("known.site.com");
        hMap.setSource("testSource");
        hMap.addGroupToAccountMapping(gMap.getName());
        
    	Configuration newConf = new Configuration();
        newConf.setIsTemplate(true);
        newConf.setSource("testSource");
        gMap = new GroupToAccountMapping(newConf, "mockGroup");
        gMap.setAccountingVoSubgroup("mock2");
        gMap.setAccountingVo("mock2");
        GroupToAccountMapping gMap2 = new GroupToAccountMapping(newConf, "mockGroup2");
        hMap.addGroupToAccountMapping(gMap2.getName());
        userGroup = new ManualUserGroup(newConf, "mockUserGroup");
        userGroup.addMember(new GridUser("/DC=org/DC=griddev/OU=People/CN=John Smith", null));
        ManualUserGroup userGroup2 = new ManualUserGroup(newConf, "mockUserGroup2");
        userGroup2.addMember(new GridUser("/DC=org/DC=griddev/OU=People/CN=Jane Doe", null));
        gMap.addUserGroup(userGroup.getName());
        gMap.addUserGroup(userGroup2.getName());
        accountMapper = new ManualAccountMapper(newConf, "mockAccountMapper2");
        gMap.addAccountMapper(accountMapper.getName());
        hMap = new CertificateHostToGroupMapping(newConf);
        hMap.setCn("known.site.com");
        hMap.addGroupToAccountMapping(gMap.getName());
        
        conf.mergeConfigurationTemplate(newConf, "known.site.com", false);

        assertEquals(conf.getGroupToAccountMapping("mockGroup").getAccountingVo(), "mock2");
        assertEquals(conf.getGroupToAccountMapping("mockGroup").getAccountingVoSubgroup(), "mock2");
        assertEquals(StringUtil.toList(conf.getGroupToAccountMapping("mockGroup").getUserGroups()).size(), 2);
        assertNotNull(conf.getGroupToAccountMapping("mockGroup2"));
        assertNotNull(conf.getUserGroup("mockUserGroup"));
        assertNotNull(conf.getUserGroup("mockUserGroup2"));
        assertNotNull(conf.getAccountMapper("mockAccountMapper"));
        assertNotNull(conf.getAccountMapper("mockAccountMapper2"));
    }    
}