/*
 * ConfigurationToolkitTest.java
 * JUnit based test
 *
 * Created on June 4, 2004, 2:38 PM
 */

package gov.bnl.gums.configuration;

import gov.bnl.gums.account.*;
import gov.bnl.gums.groupToAccount.*;
import gov.bnl.gums.hostToGroup.*;
import gov.bnl.gums.userGroup.*;
import gov.bnl.gums.util.StringUtil;

import java.io.FileInputStream;
import java.io.StringReader;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import junit.framework.*;
import org.apache.commons.digester.Digester;
import org.xml.sax.SAXParseException;

/**
 *
 * @author carcassi
 */
public class ConfigurationToolkitTest extends TestCase {
    
    ConfigurationToolkit toolkit = new ConfigurationToolkit();
    
    public ConfigurationToolkitTest(java.lang.String testName) {
        super(testName);
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(ConfigurationToolkitTest.class);
        return suite;
    }
    
    public void testGumsAttributes() throws Exception {
        String xml = 
        "<gums allowGridmapFiles='true'>" +
		"</gums>";
        Digester digester = ConfigurationToolkit.retrieveDigester();
        Configuration conf = new Configuration();
        digester.push(conf);
        Configuration acc = (Configuration) digester.parse(new StringReader(xml));
        assertSame(conf, acc);
        assertTrue(acc.isAllowGridmapFiles());
    }
    
    public void testSimpleUserGroup() throws Exception {
        String xml = 
        "<gums version='1.4'>" +
	        "<userGroups>" +
	        	"<manualUserGroup name='myGroupA' access='read self'/>" +
	        	"<manualUserGroup name='myGroupB' access='read all'/>" +
	        	"<manualUserGroup name='myGroupC' access='write'/>" +
	        "</userGroups>" +
        "</gums>";
        Digester digester = ConfigurationToolkit.retrieveDigester();
        Configuration conf = new Configuration();
        digester.push(conf);
        Configuration acc = (Configuration) digester.parse(new StringReader(xml));
        assertSame(conf, acc);
        assertTrue(acc.getUserGroups().size()==3);
        
        assertTrue( ((UserGroup)acc.getWriteUserGroups().get(0)).getName().equals("myGroupC") );
        
        assertTrue( ((UserGroup)acc.getReadAllUserGroups().get(0)).getName().equals("myGroupB") );
        assertTrue( ((UserGroup)acc.getReadAllUserGroups().get(1)).getName().equals("myGroupC") );
        
        assertTrue( ((UserGroup)acc.getReadSelfUserGroups().get(0)).getName().equals("myGroupA") );
        assertTrue( ((UserGroup)acc.getReadSelfUserGroups().get(1)).getName().equals("myGroupB") );
        assertTrue( ((UserGroup)acc.getReadSelfUserGroups().get(2)).getName().equals("myGroupC") );
    }

    public void testUserGroupWrongProperty() throws Exception {
        String xml = 
        "<gums version='1.4'>" +
	        "<userGroups>" +
	        	"<manualUserGroup name='myGroup' wrongProp='blah'/>" +
	        "</userGroups>" +
        "</gums>";
        Digester digester = ConfigurationToolkit.retrieveDigester();
        Configuration conf = new Configuration();
        digester.push(conf);
        try {
            @SuppressWarnings("unused")
			Configuration acc = (Configuration) digester.parse(new StringReader(xml));
        } catch (SAXParseException e) {
            assertEquals(java.lang.NoSuchMethodException.class, e.getException().getClass());
            return;
        }
        fail("Configuration had a wrong property but parsing didn't fail");
    }
    
    public void testVOMSUserGroup() throws Exception {
        String xml = 
        "<gums version='1.4'>" +
	        "<vomsServers>" +
	        	"<vomsServer name='myvo' persistenceFactory='mysql'/>" +
	        "</vomsServers>" +
        "</gums>";
        Digester digester = ConfigurationToolkit.retrieveDigester();
        Configuration conf = new Configuration();
        digester.push(conf);
        Configuration acc = (Configuration) digester.parse(new StringReader(xml));
        assertSame(conf, acc);
    }    
    
    public void testGroupToAccountMapping() throws Exception {
        String xml = 
        "<gums version='1.4'>" +
	        "<userGroups>" +
	        	"<manualUserGroup name='myUserGroup'/>" +
	        "</userGroups>" +
	        "<accountMappers>" +
	        	"<manualAccountMapper name='myAccountMapper1'/>" +
	        	"<accountPoolMapper name='myAccountMapper2' accountPool='myPool'/>" +
	        "</accountMappers>" +        
	        "<groupToAccountMappings>" +
	        	"<groupToAccountMapping name='myMapping' userGroups='myUserGroup' accountMappers='myAccountMapper1, myAccountMapper2'/>" +
	        "</groupToAccountMappings>" +
        "</gums>";
        Digester digester = ConfigurationToolkit.retrieveDigester();
        Configuration conf = new Configuration();
        digester.push(conf);
        Configuration acc = (Configuration) digester.parse(new StringReader(xml));
        assertSame(conf, acc);
        GroupToAccountMapping gMap = (GroupToAccountMapping) conf.getGroupToAccountMappings().get("myMapping");
        assertNotNull(gMap);
        List<String> g2amUserGroups = StringUtil.toList(gMap.getUserGroups());
        Iterator<String> it = g2amUserGroups.iterator();
        ManualUserGroup group = (ManualUserGroup) conf.getUserGroup( (String)it.next() );
        assertEquals("myUserGroup", group.getName());
        List<String> g2amAccountMappers = StringUtil.toList(gMap.getAccountMappers());
        it = g2amAccountMappers.iterator();
        AccountMapper accountMapper = (ManualAccountMapper) conf.getAccountMapper( (String)it.next() );
        assertEquals("myAccountMapper1", accountMapper.getName());
        accountMapper = (AccountPoolMapper) conf.getAccountMapper( (String)it.next() );
        assertEquals("myAccountMapper2", accountMapper.getName());
    }
    
    public void testHostGroup() throws Exception {
        String xml = 
        "<gums version='1.4'>" +
	        "<userGroups>" +
	        	"<manualUserGroup name='myUserGroup' access='read all'/>" +
	        "</userGroups>" +
	        "<accountMappers>" +
	        	"<manualAccountMapper name='myAccountMapper1'/>" +
	        	"<manualAccountMapper name='myAccountMapper2'/>" +
	        "</accountMappers>" +        
	        "<groupToAccountMappings>" +
	        	"<groupToAccountMapping name='atlas' userGroups='myUserGroup' accountMappers='myAccountMapper1, myAccountMapper2'/>" +
	        	"<groupToAccountMapping name='rhic' userGroups='myUserGroup' accountMappers='myAccountMapper1'/>" +
	        	"<groupToAccountMapping name='star' userGroups='myUserGroup' accountMappers='myAccountMapper1'/>" +
	        "</groupToAccountMappings>" +
	        "<hostToGroupMappings>" +
        		"<certificateHostToGroupMapping dn='atlas*.*.bnl.gov' groupToAccountMappings='atlas'/>" +
        		"<certificateHostToGroupMapping cn='*.rhic.bnl.gov' groupToAccountMappings='rhic, star'/>" +
        	"</hostToGroupMappings>" +
        "</gums>";
        Digester digester = ConfigurationToolkit.retrieveDigester();
        Configuration conf = new Configuration();
        digester.push(conf);
        Configuration acc = null;
		acc = (Configuration) digester.parse(new StringReader(xml));
        assertSame(conf, acc);
        List<HostToGroupMapping> hostToGroupMappings = conf.getHostToGroupMappings();
        assertNotNull(hostToGroupMappings);
        assertEquals(2, hostToGroupMappings.size());
        CertificateHostToGroupMapping hostToGroupMapping = (CertificateHostToGroupMapping) hostToGroupMappings.get(0);
        assertTrue(hostToGroupMapping.isInGroup("atlasgrid01.usatlas.bnl.gov"));
        assertFalse(hostToGroupMapping.isInGroup("stargrid01.rhic.bnl.gov"));
        assertEquals(1, StringUtil.toList(hostToGroupMapping.getGroupToAccountMappings()).size());
        assertEquals("atlas", (String)StringUtil.toList(hostToGroupMapping.getGroupToAccountMappings()).get(0));
        CertificateHostToGroupMapping certificateHostToGroupMapping = (CertificateHostToGroupMapping) hostToGroupMappings.get(1);
        assertTrue(certificateHostToGroupMapping.isInGroup("/DC=org/DC=griddev/OU=Services/CN=stargrid01.rhic.bnl.gov"));
        assertFalse(certificateHostToGroupMapping.isInGroup("atlasgrid01.usatlas.bnl.gov"));
        assertEquals(2, StringUtil.toList(certificateHostToGroupMapping.getGroupToAccountMappings()).size());
        assertEquals("rhic", (String)StringUtil.toList(certificateHostToGroupMapping.getGroupToAccountMappings()).get(0));
    }
    
    public void testTransform1_1() throws Exception {
    	URL url = getClass().getClassLoader().getResource("gums.config.1.1");
    	FileInputStream fileInputStream = new FileInputStream(url.getPath());
		StringBuffer configBuffer = new StringBuffer();
		try {
			int ch;
			while ((ch = fileInputStream.read()) != -1)
				configBuffer.append((char)ch);
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		} finally {
			fileInputStream.close();
		}
		ConfigurationToolkit.parseConfiguration(configBuffer.toString(), false);
    }

    public void testTransform1_3() throws Exception {
    	URL url = getClass().getClassLoader().getResource("gums.config.1.3");
    	FileInputStream fileInputStream = new FileInputStream(url.getPath());
		StringBuffer configBuffer = new StringBuffer();
		try {
			int ch;
			while ((ch = fileInputStream.read()) != -1)
				configBuffer.append((char)ch);
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		} finally {
			fileInputStream.close();
		}
    	ConfigurationToolkit.parseConfiguration(configBuffer.toString(), false);
    }
    
}
