package gov.bnl.gums.persistence;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import gov.bnl.gums.GridUser;
import gov.bnl.gums.Mapping;
import gov.bnl.gums.SiteUser;
import gov.bnl.gums.account.AccountMapper;
import gov.bnl.gums.account.AccountPoolMapper;
import gov.bnl.gums.account.GroupAccountMapper;
import gov.bnl.gums.account.LdapAccountMapper;
import gov.bnl.gums.account.LdapAccountPoolMapper;
import gov.bnl.gums.account.ManualAccountMapper;
import gov.bnl.gums.configuration.Configuration;
import gov.bnl.gums.groupToAccount.GroupToAccountMapping;
import gov.bnl.gums.hostToGroup.CertificateHostToGroupMapping;
import gov.bnl.gums.userGroup.LDAPUserGroup;
import gov.bnl.gums.userGroup.ManualUserGroup;
import gov.bnl.gums.userGroup.UserGroup;
import gov.bnl.gums.userGroup.VOMSUserGroup;

import junit.framework.*;

public class HibernatePersistenceEngineTest extends TestCase {
	public HibernatePersistenceEngineTest() {
		super("test");
	}
	
    public static Test suite() {
        TestSuite suite = new TestSuite(HibernatePersistenceEngineTest.class);
        return suite;
    }

	// for testing
	static protected Properties getTestHibernateProperties() {
		PropertyResourceBundle prop = (PropertyResourceBundle) ResourceBundle.getBundle("hibernate");
		Properties properties = new Properties();
		Enumeration<String> keys = prop.getKeys();
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			properties.setProperty(key, prop.getString(key));
		}
		return properties;
	}
    
	static public void testBasicConfigurationRetrievalAndStorage() throws Exception {
		HibernatePersistenceEngine persEngine = null;
		try {
			Configuration conf = new Configuration();
			
			ManualAccountMapper accountMapper1 = new ManualAccountMapper(conf, "test1");
				accountMapper1.createMapping("/DN=test/CN=test1", new SiteUser("test1"));
				accountMapper1.getMappings().get(0).setState("assigned");
			ManualAccountMapper accountMapper2 = new ManualAccountMapper(conf, "test2");
				accountMapper2.createMapping("/DN=test/CN=test2", new SiteUser("test2"));
				accountMapper2.createMapping("/DN=test/CN=test2", new SiteUser("test2b"));
			AccountMapper accountMapper3 = new AccountPoolMapper(conf, "test3");
			AccountMapper accountMapper4 = new LdapAccountMapper(conf, "test4");
			AccountMapper accountMapper5 = new LdapAccountPoolMapper(conf, "test5");
			AccountMapper accountMapper6 = new GroupAccountMapper(conf, "test6");

			ManualUserGroup userGroup1 = new ManualUserGroup(conf, "test1");
				userGroup1.setMembersUri("http://blah");
				userGroup1.addMember(new GridUser("/DN=test/CN=test1", "/test1", "test1@test1"));
			ManualUserGroup userGroup2 = new ManualUserGroup(conf, "test2");
				userGroup2.addMember(new GridUser("/DN=test/CN=test2"));
				userGroup2.addMember(new GridUser("/DN=test/CN=test2b"));
				userGroup2.setAccess("read all");
			VOMSUserGroup userGroup3 = new VOMSUserGroup(conf, "test3");
			userGroup3.setVomsServerUrls("http://blah1, http://blah2");
			UserGroup userGroup4 = new LDAPUserGroup(conf, "test4");
			
			GroupToAccountMapping g2AMapping = new GroupToAccountMapping(conf, "test");
			g2AMapping.addUserGroup(userGroup1.getName());
			g2AMapping.addUserGroup(userGroup2.getName());
			g2AMapping.addUserGroup(userGroup3.getName());
			g2AMapping.addUserGroup(userGroup4.getName());
			g2AMapping.addAccountMapper(accountMapper1.getName());
			g2AMapping.addAccountMapper(accountMapper2.getName());
			g2AMapping.addAccountMapper(accountMapper3.getName());
			g2AMapping.addAccountMapper(accountMapper4.getName());
			g2AMapping.addAccountMapper(accountMapper5.getName());
			g2AMapping.addAccountMapper(accountMapper6.getName());
			
			CertificateHostToGroupMapping h2GMapping = new CertificateHostToGroupMapping(conf);
			//h2GMapping.setCn("test");
			h2GMapping.setName("test");
			h2GMapping.setSource("test source");
			h2GMapping.addGroupToAccountMapping(g2AMapping.getName());
			
			conf.setBannedUserGroups("test1, test2");
			conf.setMergeURIs("http://test, http://test2");

			persEngine = new HibernatePersistenceEngine(HibernatePersistenceEngineTest.getTestHibernateProperties());
			persEngine.beginTransaction();
			persEngine.setConfiguration(conf);
			persEngine.commit();
			
			Configuration retrievedConf = persEngine.retrieveCurrentConfiguration();
			
			assertEquals(conf.getAccountMapper("test1"),retrievedConf.getAccountMapper("test1"));
				assertEquals(((ManualAccountMapper)retrievedConf.getAccountMapper("test1")).getMappings().get(0),new Mapping("/DN=test/CN=test1", new SiteUser("test1"), "assigned"));
				assertEquals(((ManualAccountMapper)retrievedConf.getAccountMapper("test1")).getMappings().size(), 1);
			assertEquals(conf.getAccountMapper("test2"),retrievedConf.getAccountMapper("test2"));
				assertEquals(((ManualAccountMapper)retrievedConf.getAccountMapper("test2")).getMappings().get(0),new Mapping("/DN=test/CN=test2", new SiteUser("test2")));
				assertEquals(((ManualAccountMapper)retrievedConf.getAccountMapper("test2")).getMappings().get(1),new Mapping("/DN=test/CN=test2", new SiteUser("test2b")));
				assertEquals(((ManualAccountMapper)retrievedConf.getAccountMapper("test2")).getMappings().size(), 2);
			assertEquals(conf.getAccountMapper("test3"),retrievedConf.getAccountMapper("test3"));
			assertEquals(conf.getAccountMapper("test4"),retrievedConf.getAccountMapper("test4"));
			assertEquals(conf.getAccountMapper("test5"),retrievedConf.getAccountMapper("test5"));
			assertEquals(conf.getAccountMapper("test6"),retrievedConf.getAccountMapper("test6"));

			assertEquals(conf.getUserGroup("test1"), retrievedConf.getUserGroup("test1"));
				assertEquals(((ManualUserGroup)retrievedConf.getUserGroup("test1")).getMembers().iterator().next(), new GridUser("/DN=test/CN=test1", "/test1", "test1@test1"));
				assertEquals(((ManualUserGroup)retrievedConf.getUserGroup("test1")).getMembers().size(), 1);
			assertEquals(conf.getUserGroup("test2"), retrievedConf.getUserGroup("test2"));
				Iterator<GridUser> it = ((ManualUserGroup)retrievedConf.getUserGroup("test2")).getMembers().iterator();
				assertEquals(it.next(), new GridUser("/DN=test/CN=test2b"));
				assertEquals(it.next(), new GridUser("/DN=test/CN=test2"));
				assertEquals(((ManualUserGroup)retrievedConf.getUserGroup("test2")).getMembers().size(), 2);
			assertEquals(conf.getUserGroup("test3"), retrievedConf.getUserGroup("test3"));
			assertEquals(conf.getUserGroup("test4"), retrievedConf.getUserGroup("test4"));
			
			assertEquals(conf.getGroupToAccountMapping("test"), retrievedConf.getGroupToAccountMapping("test"));
			
			assertEquals(conf.getHostToGroupMapping("test"), retrievedConf.getHostToGroupMapping("test"));

			assertEquals(conf.getSource(), retrievedConf.getSource());
			assertEquals(conf.getBannedUserGroups(), retrievedConf.getBannedUserGroups());

			// add another configuration
			persEngine.beginTransaction();
			persEngine.setConfiguration(new Configuration());
			persEngine.commit();
		} catch (Exception e) {
			e.printStackTrace();
			throw e;		
		}
	}
}
